/*----------------------------------------------------------------------------*/
/* Copyright (c) 2017-2018 FIRST. All Rights Reserved.                        */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

package team.frc.morpheus.robot;

import edu.wpi.first.wpilibj.smartdashboard.SendableChooser;
import edu.wpi.first.wpilibj.smartdashboard.SmartDashboard;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SequentialCommandGroup;
import team.frc.morpheus.robot.commands.drivebase.DriverControl;
import team.frc.morpheus.robot.commands.drivebase.ExecuteCSVTrajectory;
import team.frc.morpheus.robot.commands.drivebase.ExecuteReverseCSVTrajectory;
import team.frc.morpheus.robot.commands.drivebase.features.AssistedEnhanced;
import team.frc.morpheus.robot.subsystems.DriveBase;
import team.frc.morpheus.util.input.Gamepad360;
import team.frc.morpheus.util.input.LambdaButton;

import java.io.File;

/**
 * This class is where the bulk of the robot should be declared.  Since Command-based is a
 * "declarative" paradigm, very little robot logic should actually be handled in the {@link Robot}
 * periodic methods (other than the scheduler calls).  Instead, the structure of the robot
 * (including subsystems, commands, and button mappings) should be declared here.
 */
public class RobotContainer {
    private final Gamepad360 driver, operator;
    private final DriverParams driverParams;
    private final SendableChooser<Command> autoChooser;
    private final DriveBase driveBase;

    public RobotContainer() {
        driver = new Gamepad360(0);
        operator = new Gamepad360(1);
        final DriverParams brianna = new DriverParams(driver::getLeftStick, new AssistedEnhanced(driver::getRightTrigger),
                DriverControl.Features.buildSlowMode(driver::getButtonLB, 0.7, 0.7));
        final DriverParams garrett = new DriverParams(driver::getLeftStick, new AssistedEnhanced(driver::getRightTrigger),
                DriverControl.Features.buildSlowMode(driver::getButtonLB, 0.55, 0.8));
        final DriverParams gabe = new DriverParams(driver::getLeftStick, new AssistedEnhanced(driver::getRightTrigger),
                DriverControl.Features.buildSlowMode(driver::getButtonLB, 0.55, 0.8));
        driverParams = gabe;

        autoChooser = new SendableChooser<>();

        driveBase = DriveBase.getInstance();

        configureButtonBindings();
        configureAutonomousModes();
    }

    /**
     * Sets up button bindings
     */
    private void configureButtonBindings() {

    }

    /**
     * Adds autonomous CommandGroups to the autoChooser
     */
    private void configureAutonomousModes() {
        /* Looks deprecated but is still required for some reason (as of 2019)! */
        autoChooser.addDefault("No-op Auto", () -> null); // no-op auto; put your auto here to run before teleop
        SmartDashboard.putData("Autonomous Chooser", autoChooser);
    }

    /**
     * Returns the autonomous Command selected on the SmartDashBoard / ShuffleBoard
     *
     * @return the selected autonomous Command
     */
    public Command getAutonomousCommand() {
        return autoChooser.getSelected();
    }

    /**
     * Returns an instance of DriverControl.
     * This is the place to add features if needed.
     *
     * @return an instance of DriverControl
     */
    public Command getDriverControl() {
        return driverParams.getDriverControl();
    }
}

