package team.frc.morpheus.robot.commands.drivebase;

import edu.wpi.first.wpilibj.Timer;
import edu.wpi.first.wpilibj2.command.CommandBase;
import team.frc.morpheus.robot.Robot;
import team.frc.morpheus.robot.subsystems.DriveBase;
import team.frc.morpheus.util.input.Coordinate;

import java.util.Optional;

/**
 * Takes inches not feet
 */

public class DriveStraight extends CommandBase {
    DriveBase driveBase;
    double targetPosition, currentPosition;
    boolean reverse = false;
    double inches;
    Timer timer;

    Optional<Double> timeout = Optional.empty();

    public DriveStraight(double inches) {
        driveBase = DriveBase.getInstance();
        hasRequirement(driveBase);
        if (inches < 0) reverse = true;
        this.inches = inches;
    }

    public DriveStraight(double inches, double timeout) {
        driveBase = DriveBase.getInstance();
        hasRequirement(driveBase);
        if (inches < 0) reverse = true;
        this.inches = inches;
        this.timeout = Optional.of(timeout);
        timer = new Timer();
    }

    // Called just before this Command runs the first time
    @Override
    public void initialize() {
        targetPosition = inches + driveBase.getLeftPosition();
        driveBase.set(new Coordinate.Cartesian(0, reverse ? -0.42 : 0.42));
        if (timeout.isPresent()) {
            timer.start();
        }
    }

    // Called repeatedly when this Command is scheduled to run
    @Override
    public void execute() {
        currentPosition = driveBase.getLeftPosition();
        System.out.println("This is left: " + driveBase.getLeftPosition());
        System.out.println("This is right: " + driveBase.getRightPosition());
    }

    // Make this return true when this Command no longer needs to run execute()
    @Override
    public boolean isFinished() {
        if (timeout.isPresent()) {
            if (reverse) {
                return currentPosition <= targetPosition || timer.get() > timeout.get();
            } else {
                return currentPosition >= targetPosition || timer.get() > timeout.get();
            }
        } else {
            if (reverse) {
                return currentPosition <= targetPosition;
            } else {
                return currentPosition >= targetPosition;
            }
        }
    }

    // Called once after isFinished returns true
    @Override
    public void end(boolean interrupted) {
        driveBase.directSet(0, 0);
        if (timeout.isPresent()) {
            timer.stop();
        }
    }
}