package team.frc.morpheus.robot.commands.drivebase.features;

import team.frc.morpheus.robot.Constants;
import team.frc.morpheus.robot.commands.drivebase.DriverControl;
import team.frc.morpheus.util.input.Coordinate;
import team.frc.morpheus.vision.limelight.DoubleTargetTrackerLimelight;

import java.util.function.DoubleSupplier;

public class AssistedEnhanced implements DriverControl.Feature {
    private double accumulator, oldError;
    private DoubleSupplier correctionExtentSource;

    public AssistedEnhanced(DoubleSupplier correctionExtentSource) {
        this.correctionExtentSource = correctionExtentSource;
        accumulator = 0;
        oldError = 0;
    }

    @Override
    public Coordinate apply(Coordinate in) {
        double correctionExtent = correctionExtentSource.getAsDouble();
        double error = DoubleTargetTrackerLimelight.getLRAdjustment();
        double correction = error * Constants.ASSISTED_DRIVING_KP + accumulator * Constants.ASSISTED_DRIVING_KI + (error - oldError) * Constants.ASSISTED_DRIVING_KD;
        accumulator += error;
        oldError = error;
        return new Coordinate.Cartesian(in.x() + correctionExtent * correction, in.y());

    }
}
